#include "MainWindow.h"

#include <QMenu>
#include <QMenuBar>
#include <QToolBar>
#include <QStatusBar>
#include <QIcon>
#include <QPixmap>
#include <QDockWidget>
#include <QFileDialog>
#include <QFile>
#include <QTextStream>

#include <fstream>

#include "AboutDialog.h"

MainWindow::MainWindow() : QMainWindow(NULL) {
  resize(600, 400);
  setWindowTitle("MainWindow");

  InitActions();
  InitMenuBar();
  InitToolBar();
  InitStatusBar();
  InitCodeEditor();
  InitProcess();

  QDockWidget* dock = new QDockWidget("Log Window", this);
  log = new QTextEdit(this);
  log->setReadOnly(true);
  dock->setWidget(log);
  dock->setMaximumHeight(150);
  addDockWidget(Qt::BottomDockWidgetArea, dock);

  show();
}

void MainWindow::InitActions() {
  //Menu "File"
  actFileOpen = new QAction(QIcon("/home/student/KRPO_Lab_0x07/task_02/open.png"), "&Open", this);
  connect(actFileOpen, SIGNAL(triggered()), this, SLOT(OnMenuFileOpen()));
  actFileOpen->setStatusTip("Open file and load data");
  actFileOpen->setToolTip("Open file");
  
  actFileExit = new QAction(QIcon("/home/student/KRPO_Lab_0x07/task_02/exit.png"), "E&xit", this);
  connect(actFileExit, SIGNAL(triggered()), this, SLOT(OnMenuFileExit()));
  actFileExit->setStatusTip("Exit this app");
  actFileExit->setToolTip("Exit app");
  //Menu "View"
  actViewToolbar = new QAction("&Toolbar", this);
  connect(actViewToolbar, SIGNAL(triggered()), this, SLOT(OnMenuViewToolbar()));
  actViewToolbar->setCheckable(true);
  actViewToolbar->setChecked(true);
  
  actViewStatusbar = new QAction("&Statusbar", this);
  connect(actViewStatusbar, SIGNAL(triggered()), this, SLOT(OnMenuViewStatusbar()));
  actViewStatusbar->setCheckable(true);
  actViewStatusbar->setChecked(true);

  //Menu "Run"
  actRunExecute = new QAction(QIcon("/home/student/KRPO_Lab_0x07/task_02/run.png"), "&Execute", this);
  connect(actRunExecute, SIGNAL(triggered()), this, SLOT(OnMenuRunExecute()));

  //Menu "Help"
  actHelpAbout = new QAction(QIcon("about.png"), "&About", this);
  connect(actHelpAbout, SIGNAL(triggered()), this, SLOT(OnMenuHelpAbout()));
}

void MainWindow::InitMenuBar() {
  QMenu *menuFile = new QMenu("File");
  menuFile->addAction(actFileOpen);
  menuFile->addSeparator();
  menuFile->addAction(actFileExit);
  menuBar()->addMenu(menuFile);

  QMenu *menuView = new QMenu("View");
  menuView->addAction(actViewToolbar);
  menuView->addAction(actViewStatusbar);
  menuBar()->addMenu(menuView);

  QMenu *menuRun = new QMenu("Run");
  menuRun->addAction(actRunExecute);
  menuBar()->addMenu(menuRun);

  QMenu *menuHelp = new QMenu("Help");
  menuHelp->addAction(actHelpAbout);
  menuBar()->addMenu(menuHelp);
}

void MainWindow::InitToolBar() {
  toolBar = new QToolBar(this);
  toolBar->setIconSize(QSize(32, 32));
  toolBar->addAction(actFileOpen);
  toolBar->addSeparator();
  toolBar->addAction(actRunExecute);
  toolBar->addSeparator();
  toolBar->addAction(actFileExit);
  addToolBar(toolBar);
}

void MainWindow::InitStatusBar() {
  statusBar = new QStatusBar(this);
  setStatusBar(statusBar);
  statusBar->showMessage("Welcome");
  statusBar->setStatusTip("StatusBar");
}

void MainWindow::InitCodeEditor() {
  codeEditor = new QTextEdit(this);
  highlighter = new Highlighter(codeEditor->document());
  setCentralWidget(codeEditor);
}

void MainWindow::InitProcess() {
  process = new QProcess(this);
  process->setProcessChannelMode(QProcess::MergedChannels);
  connect(process, SIGNAL(started()), this, SLOT(OnProcessStarted()));
  connect(process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(OnProcessFinished(int, QProcess::ExitStatus)));
  connect(process, SIGNAL(readyReadStandardOutput()), this, SLOT(OnReadStandardOutput()));
}

void MainWindow::OnMenuFileOpen() {
  log->append("Trying to open file...\n");
}

void MainWindow::OnMenuFileExit() {
  close();
}

void MainWindow::OnMenuViewToolbar() {
  toolBar->setVisible(actViewToolbar->isChecked());
}

void MainWindow::OnMenuViewStatusbar() {
  statusBar->setVisible(actViewStatusbar->isChecked());
}

void MainWindow::OnMenuRunExecute() {
  process->start("run.exe");
}

void MainWindow::OnMenuHelpAbout() {
  AboutDialog dlg(this);
  dlg.exec();
}

void MainWindow::OnProcessStarted() {
}

void MainWindow::OnProcessFinished(int exitCode, QProcess::ExitStatus) {
}

void MainWindow::OnReadStandardOutput() {
}

